import unittest
import datetime
from kev.validators import (RequiredValidator,DateTimeValidator,
                            DateValidator,FloatValidator,IntegerValidator,
                            MaxLengthValidator,MinLengthValidator,
                            MaxValueValidator,MinValueValidator,
                            StringValidator,ValidationException,
                            BooleanValidator
                            )
from kev.document import Document
from kev.properties import CharProperty

class Frog(Document):
    name = CharProperty()


class Dog(Document):
    name = CharProperty()

    class Meta(object):
        doc_type = 'animal'


class ValidatorsTestCase(unittest.TestCase):

    def test_required_validator(self):
        with self.assertRaises(ValidationException) as vm:
            RequiredValidator().validate(None,'first_name')
        self.assertEqual(str(vm.exception), 'first_name: This value is required')
        #Test with valid input
        RequiredValidator().validate('First Name','first_name')

    def test_datetime_validator(self):
        with self.assertRaises(ValidationException) as vm:
            DateTimeValidator().validate(datetime.date.today(), 'date_created')
        self.assertEqual(str(vm.exception),
            'date_created: This value should be a valid datetime object.')
        # Test with valid input
        DateTimeValidator().validate(datetime.datetime.now(),'date_created')

    def test_date_validator(self):
        with self.assertRaises(ValidationException) as vm:
            DateValidator().validate('not a date', 'date_created')
        self.assertEqual(str(vm.exception),
                          'date_created: This value should be a valid date object.')
        # Test with valid input
        DateValidator().validate(datetime.date.today(),'date_created')

    def test_float_validator(self):
        with self.assertRaises(ValidationException) as vm:
            FloatValidator().validate(1, 'no_packages')
        self.assertEqual(str(vm.exception),
                          'no_packages: This value should be a float.')
        # Test with valid input
        FloatValidator().validate(1.3,'no_packages')

    def test_integer_validator(self):
        with self.assertRaises(ValidationException) as vm:
            IntegerValidator().validate(1.2, 'no_packages')
        self.assertEqual(str(vm.exception),
                          'no_packages: This value should be an integer')
        # Test with valid input
        IntegerValidator().validate(1, 'no_packages')

    def test_max_length_validator(self):
        with self.assertRaises(ValidationException) as vm:
            MaxLengthValidator(2).validate('123', 'no_packages')
        self.assertEqual(str(vm.exception),
            'no_packages: This value should have a length lesser than 2. Currently 123')
        # Test with valid input
        MaxLengthValidator(2).validate('12', 'no_packages')

    def test_min_length_validator(self):
        with self.assertRaises(ValidationException) as vm:
            MinLengthValidator(2).validate('1', 'no_packages')
        self.assertEqual(str(vm.exception),
            'no_packages: This value should have a length greater than 2. Currently 1')
        # Test with valid input
        MinLengthValidator(2).validate('123', 'no_packages')

    def test_max_value_validator(self):
        with self.assertRaises(ValidationException) as vm:
            MaxValueValidator(2).validate(3, 'no_packages')
        self.assertEqual(str(vm.exception),
            'no_packages: This value should have a value lesser than 2. Currently 3')
        # Test with valid input
        MaxValueValidator(2).validate(1, 'no_packages')

    def test_min_value_validator(self):
        with self.assertRaises(ValidationException) as vm:
            MinValueValidator(2).validate(1, 'no_packages')
        self.assertEqual(str(vm.exception),
                          'no_packages: This value should have a value greater than 2. Currently 1')
        # Test with valid input
        MinValueValidator(2).validate(3, 'no_packages')

    def test_string_validator(self):
        with self.assertRaises(ValidationException) as vm:
            StringValidator().validate(1, 'last_name')
        self.assertEqual(str(vm.exception),
                          'last_name: This value should be a string')
        # Test with valid input
        StringValidator().validate('Jones', 'last_name')

    def test_boolean_validator(self):
        with self.assertRaises(ValidationException) as vm:
            BooleanValidator().validate(1, 'last_name')
        self.assertEqual(str(vm.exception),
                          'last_name: This value should be True or False.')
        # Test with valid input
        BooleanValidator().validate(True, 'last_name')
        BooleanValidator().validate(False, 'last_name')

if __name__ == '__main__':
    unittest.main()


import unittest
import datetime

from kev import (Document,CharProperty,DateTimeProperty,
                 DateProperty,BooleanProperty,IntegerProperty,
                 FloatProperty)
from kev.exceptions import ValidationException, QueryError
from kev.query import combine_list, combine_dicts
from kev.testcase import kev_handler,KevTestCase


class TestDocument(Document):
    name = CharProperty(required=True,unique=True,min_length=5,max_length=20)
    last_updated = DateTimeProperty(auto_now=True)
    date_created = DateProperty(auto_now_add=True)
    is_active = BooleanProperty(default_value=True)
    no_subscriptions = IntegerProperty(default_value=1,min_value=1,max_value=20)
    gpa = FloatProperty()

    def __unicode__(self):
        return self.name

    class Meta:
        use_db = 's3'
        handler = kev_handler


class BaseTestDocumentSlug(TestDocument):
    slug = CharProperty(required=True,unique=True)
    email = CharProperty(required=True,unique=True)
    city = CharProperty(required=True,index=True)


class S3TestDocumentSlug(BaseTestDocumentSlug):
    class Meta:
        use_db = 's3'
        handler = kev_handler


class RedisTestDocumentSlug(BaseTestDocumentSlug):
    class Meta:
        use_db = 'redis'
        handler = kev_handler


class DocumentTestCase(KevTestCase):

    def test_default_values(self):
        obj = TestDocument(name='Fred')
        self.assertEqual(obj.is_active,True)
        self.assertEqual(obj._doc.get('is_active'),True)
        self.assertEqual(obj.date_created,datetime.date.today())
        self.assertEqual(obj._doc.get('date_created'), datetime.date.today())
        self.assertEqual(type(obj.last_updated),datetime.datetime)
        self.assertEqual(type(obj._doc.get('last_updated')), datetime.datetime)
        self.assertEqual(obj.no_subscriptions,1)
        self.assertEqual(obj._doc.get('no_subscriptions'), 1)
        self.assertEqual(obj.gpa,None)
        self.assertFalse('gpa' in obj._doc)

    def test_get_unique_props(self):
        obj = S3TestDocumentSlug(name='Brian',slug='brian',email='brian@host.com',
                                 city='Greensboro',gpa=4.0)
        self.assertEqual(obj.get_unique_props().sort(),['name','slug','email'].sort())

    def test_set_indexed_prop(self):
        obj = S3TestDocumentSlug(name='Brian', slug='brian', email='brian@host.com',
                                 city='Greensboro', gpa=4.0)
        obj.name = 'Tariq'
        self.assertEqual(obj._index_change_list,['s3testdocumentslug:indexes:name:brian'])

    def test_validate_valid(self):
        t1 = TestDocument(name='DNSly',is_active=False,no_subscriptions=2,gpa=3.5)
        t1.save()

    def test_validate_boolean(self):
        t2 = TestDocument(name='Google', is_active='Gone', gpa=4.0)
        with self.assertRaises(ValidationException) as vm:
            t2.save()
        self.assertEqual(str(vm.exception),
                          'is_active: This value should be True or False.')

    def test_validate_datetime(self):
        t2 = TestDocument(name='Google',gpa=4.0,last_updated='today')
        with self.assertRaises(ValidationException) as vm:
            t2.save()
        self.assertEqual(str(vm.exception),
                          'last_updated: This value should be a valid datetime object.')

    def test_validate_date(self):
        t2 = TestDocument(name='Google', gpa=4.0, date_created='today')
        with self.assertRaises(ValidationException) as vm:
            t2.save()
        self.assertEqual(str(vm.exception),
                          'date_created: This value should be a valid date object.')

    def test_validate_integer(self):
        t2 = TestDocument(name='Google', gpa=4.0, no_subscriptions='seven')
        with self.assertRaises(ValidationException) as vm:
            t2.save()
        self.assertEqual(str(vm.exception),
                          'no_subscriptions: This value should be an integer')

    def test_validate_float(self):
        t2 = TestDocument(name='Google', gpa='seven')
        with self.assertRaises(ValidationException) as vm:
            t2.save()
        self.assertEqual(str(vm.exception),
                          'gpa: This value should be a float.')

    def test_validate_unique(self):
        t1 = TestDocument(name='Google',gpa=4.0)
        t1.save()
        t2 = TestDocument(name='Google',gpa=4.0)
        with self.assertRaises(ValidationException) as vm:
            t2.save()
        self.assertEqual(str(vm.exception),
                          'There is already a name with the value of Google')



class S3QueryTestCase(KevTestCase):

    doc_class = S3TestDocumentSlug

    def setUp(self):

        self.t1 = self.doc_class(name='Goo and Sons',slug='goo-sons',gpa=3.2,
                         email='goo@sons.com',city="Durham")
        self.t1.save()
        self.t2 = self.doc_class(name='Great Mountain', slug='great-mountain', gpa=3.2,
                                   email='great@mountain.com',city='Charlotte')
        self.t2.save()
        self.t3 = self.doc_class(name='Lakewoood YMCA', slug='lakewood-ymca', gpa=3.2,
                                   email='lakewood@ymca.com', city='Durham')
        self.t3.save()

    def test_non_unique_filter(self):
        qs = self.doc_class.objects().filter({'city':'durham'})
        self.assertEqual(2,qs.count())

    def test_objects_get_single_indexed_prop(self):
        obj = self.doc_class.objects().get({'name':self.t1.name})
        self.assertEqual(obj.slug,self.t1.slug)

    def test_get(self):
        obj = self.doc_class.get(self.t1.id)
        self.assertEqual(obj._id,self.t1._id)

    def test_flush_db(self):
        self.assertEqual(3,len(list(self.doc_class.all())))
        self.doc_class().flush_db()
        self.assertEqual(0, len(list(self.doc_class.all())))

    def test_delete(self):
        qs = self.doc_class.objects().filter({'city': 'durham'})
        self.assertEqual(2, qs.count())
        qs[0].delete()
        qs = self.doc_class.objects().filter({'city': 'durham'})
        self.assertEqual(1,qs.count())

    def test_queryset_iter(self):
        qs = self.doc_class.objects().filter({'city': 'durham'})
        for i in qs:
            self.assertIsNotNone(i.id)

    def test_queryset_chaininig(self):
        qs = self.doc_class.objects().filter(
            {'name':'Goo and Sons'}).filter({'city':'Durham'})
        self.assertEqual(1,qs.count())
        self.assertEqual(self.t1.name,qs[0].name)

    def test_objects_get_no_result(self):
        with self.assertRaises(QueryError) as vm:
            self.doc_class.objects().get({'username':'affsdfadsf'})
        self.assertEqual(str(vm.exception),
                          'This query did not return a result.')

    def test_all(self):
        qs = self.doc_class.all()
        self.assertEqual(3,len(list(qs)))

    def test_objects_get_multiple_results(self):
        with self.assertRaises(QueryError) as vm:
            self.doc_class.objects().get({'city': 'durham'})
        self.assertEqual(str(vm.exception),
            'This query should return exactly one result. Your query returned 2')

    def test_combine_list(self):
        a = [1,2,3]
        b = ['a','b','c']
        c = combine_list(a,b)
        self.assertEqual([1,2,3,'a','b','c'],c)

    def test_combine_dicts(self):
        a = {'username':'boywonder','doc_type':'goo'}
        b = {'email':'boywonder@superteam.com','doc_type':'foo'}
        c = combine_dicts(a,b)
        self.assertEqual({'username':'boywonder',
                          'email':'boywonder@superteam.com',
                          'doc_type':['goo','foo']},c)


class RedisQueryTestCase(S3QueryTestCase):

    doc_class = RedisTestDocumentSlug


if __name__ == '__main__':
    unittest.main()


import boto3
import json
import redis
import six

from kev.backends.redis.db import RedisDB

class S3DB(RedisDB):

    db_class = boto3.resource
    indexer_class = redis.StrictRedis
    backend_id = 's3'

    def __init__(self,**kwargs):

        if 'aws_secret_access_key' in kwargs and 'aws_access_key_id' in kwargs:
            boto3.Session(aws_secret_access_key=kwargs['aws_secret_access_key'],
                aws_access_key_id=kwargs['aws_access_key_id'])
        self._db = boto3.resource('s3')
        self.bucket = kwargs['bucket']

        self._indexer = self.indexer_class(**kwargs['indexer'])

    #CRUD Operation Methods

    def save(self,doc_obj):
        doc_obj, doc = self._save(doc_obj)

        self._db.Object(self.bucket, doc_obj._id).put(
                Body=json.dumps(doc))
        pipe = self._indexer.pipeline()
        pipe = self.add_to_model_set(doc_obj, pipe)
        pipe = self.add_indexes(doc_obj, doc, pipe)
        pipe = self.remove_indexes(doc_obj, pipe)
        pipe.execute()
        doc_obj._doc = doc_obj.process_doc_kwargs(doc)
        return doc_obj

    def get(self,doc_obj,doc_id):
        doc = json.loads(self._db.Object(
                self.bucket, doc_obj.get_doc_id(doc_id)).get().get('Body').read().decode())

        return doc_obj.__class__(**doc)

    def flush_db(self):
        self._indexer.flushdb()
        obj_list = self._db.Bucket(self.bucket).objects.all()
        for i in obj_list:
            i.delete()

    def delete(self, doc_obj):
        self._db.Object(self.bucket,doc_obj._id).delete()
        pipe = self._indexer.pipeline()
        pipe = self.remove_from_model_set(doc_obj, pipe)
        doc_obj._index_change_list = doc_obj.get_indexes()
        pipe = self.remove_indexes(doc_obj, pipe)
        pipe.execute()

    def all(self,cls):
        klass = cls()
        id_list = [int(id.rsplit(b':',1)[1]) for id in self._indexer.smembers('{0}:all'.format(
            klass.get_class_name()))]
        for id in id_list:
            yield self.get(klass,id)

    def evaluate(self, filters_list, doc_class):
        if len(filters_list) == 1:
            id_list = self._indexer.smembers(filters_list[0])
        else:
            id_list = self._indexer.sinter(*filters_list)
        for id in id_list:
            yield doc_class.get(self.parse_id(id))


import datetime
import time
import six

from .exceptions import ValidationException

__all__ = [
    "Validator",
    "RequiredValidator",
    "StringValidator",
    "IntegerValidator",
    "MaxValueValidator",
    "MinValueValidator",
    "MinLengthValidator",
    "MaxLengthValidator",
    "DateValidator",
    "DateTimeValidator",
    ]


class Validator(object):
        
    def validate(self,value,key):
        raise NotImplementedError


class RequiredValidator(Validator):
    
    def validate(self,value,key):
        if not value:
            raise ValidationException(
                '{0}: This value is required'.format(key)
                )


class StringValidator(Validator):
    
    def validate(self,value,key=None):
        if value and not isinstance(value, six.string_types):
            raise ValidationException(
                '{0}: This value should '
                'be a string'.format(key)
                )


class DateValidator(Validator):
    
    def validate(self, value,key=None):
        if value and isinstance(value,six.string_types):
            try:
                value = datetime.date(*time.strptime(value, '%Y-%m-%d')[:3])
            except ValueError:
                pass
        if value and not isinstance(value, datetime.date):
            raise ValidationException(
                '{0}: This value should be a valid date object.'.format(key))


class DateTimeValidator(Validator):
    
    def validate(self, value,key=None):
        if value and isinstance(value,six.string_types):
            try:
                value = value.split('.', 1)[0]  # strip out microseconds
                value = value[0:19]  # remove timezone
                value = datetime.datetime.strptime(value, '%Y-%m-%dT%H:%M:%S')
            except (IndexError,KeyError,ValueError):
                pass
        if value and not isinstance(value, datetime.datetime):
            raise ValidationException(
                '{0}: This value should be a valid datetime object.'.format(key))


class IntegerValidator(Validator):
    def validate(self,value,key=None):
        if value and not isinstance(value, int):
            raise ValidationException('{0}: This value should be an integer'.format(key))


class FloatValidator(Validator):
    def validate(self, value,key=None):
        if value and not isinstance(value, float):
            raise ValidationException('{0}: This value should be a float.'.format(key))


class MaxValueValidator(Validator):
    def __init__(self,compare_value):
        self.compare_value = compare_value
        
    def validate(self,value,key=None):
        if isinstance(value, (float,int)) and value > self.compare_value:
            raise ValidationException(
                '{0}: This value should '
                'have a value lesser than '
                '{1}. Currently {2}'.format(key,self.compare_value, value)
                )


class MinValueValidator(MaxValueValidator):
        
    def validate(self,value,key=None):
        if isinstance(value, (float,int)) and value < self.compare_value:
            raise ValidationException(
                '{0}: This value should '
                'have a value greater than '
                '{1}. Currently {2}'.format(key,self.compare_value, value)
                )


class MaxLengthValidator(Validator):
    
    def __init__(self,length):
        self.length = length
        
    def validate(self,value,key=None):
        if not isinstance(value, int) and len(value) > self.length:
            raise ValidationException(
                '{0}: This value should '
                'have a length lesser than '
                '{1}. Currently {2}'.format(key,self.length, value)
                )


class MinLengthValidator(MaxLengthValidator):
        
    def validate(self,value,key):
        if not isinstance(value, int) and len(value) < self.length:
            raise ValidationException(
                '{0}: This value should '
                'have a length greater than '
                '{1}. Currently {2}'.format(key,self.length, value)
                )


class BooleanValidator(Validator):

    def validate(self,value,key):
        if not isinstance(value,bool):
            raise ValidationException(
                '{0}: This value should be True or False.'.format(key)
            )


import os
import sys
import importlib


def import_mod(imp):
    '''
    Lazily imports a module from a string
    @param imp:
    '''
    __import__(imp,globals(),locals())
    return sys.modules[imp]
    
def import_util(imp):
    '''
    Lazily imports a utils (class,
    function,or variable) from a module) from
    a string.
    @param imp:
    '''

    mod_name,obj_name = imp.rsplit('.',1)
    mod = importlib.import_module(mod_name)
    return getattr(mod,obj_name)

def get_doc_type(klass):
    if hasattr(klass.Meta,'doc_type'):
        if klass.Meta.doc_type is not None:
            return klass.Meta.doc_type
    return klass.__name__


from setuptools import setup,find_packages
from pip.req import parse_requirements

install_reqs = parse_requirements('requirements.txt', session=False)


version = '0.5.1'
 
LONG_DESCRIPTION = """
=======================
Kev
=======================

K.E.V. (Keys, Extra Stuff, and Values) is a Python ORM for key-value stores. Currently supported
backends are Redis and a S3/Redis hybrid backend.

"""
 
setup(
    name='kev',
    version=version,
    description="""K.E.V. (Keys, Extra Stuff, and Values) is a Python ORM for key-value stores. \
        Currently supported backends are Redis and a S3/Redis hybrid backend.""",
    long_description=LONG_DESCRIPTION,
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Environment :: Web Environment",
    ],
    keywords='redis, s3',
    author='Brian Jinwright',
    author_email='opensource@ipoots.com',
    maintainer='Brian Jinwright',
    packages=find_packages(),
    
    license='Apache',
    install_requires=[str(ir.req) for ir in install_reqs],
    include_package_data=True,
    zip_safe=False,
)



from six import with_metaclass

from .properties import BaseProperty
from .query import QueryManager

BUILTIN_DOC_ATTRS = ('_id','_doc_type')

def get_declared_variables(bases, attrs):
    properties = {}
    f_update = properties.update
    attrs_pop = attrs.pop
    for variable_name, obj in list(attrs.items()):
        if isinstance(obj, BaseProperty):
            f_update({variable_name:attrs_pop(variable_name)})
        
    for base in bases:
        if hasattr(base, '_base_properties'):
            if len(base._base_properties) > 0:
                f_update(base._base_properties)
    return properties


class DeclarativeVariablesMetaclass(type):
    """
    Partially ripped off from Django's forms.
    http://code.djangoproject.com/browser/django/trunk/django/forms/forms.py
    """
    def __new__(cls, name, bases, attrs):
        attrs['_base_properties'] = get_declared_variables(bases, attrs)
        new_class = super(DeclarativeVariablesMetaclass,
            cls).__new__(cls, name, bases, attrs)

        return new_class


class BaseDocument(object):
    """
    Base class for all Kev Documents classes.
    """

    def __init__(self,**kwargs):
        self._doc = self.process_doc_kwargs(kwargs)
        self._db = self.get_db()
        if '_id' in self._doc:
            self.set_pk(self._doc['_id'])
        self._index_change_list = []

    def __repr__(self):
        return '<{class_name}: {uni}:{id} >'.format(
            class_name=self.__class__.__name__, uni=self.__unicode__(),
                                                    id=self.pk)

    def __unicode__(self):
        return '({0} Object)'.format(self.__class__.__name__)

    def __getattr__(self,name):
        if name in list(self._base_properties.keys()):
            prop = self._base_properties[name]
            return prop.get_python_value(self._doc.get(name))
    
    def __setattr__(self,name,value):
        if name in list(self._base_properties.keys()):
            if name in self.get_indexed_props() and value \
                    != self._doc.get(name) and self._doc.get(name) != None:
                    self._index_change_list.append(
                        self.get_index_name(name,self._doc[name]))
            self._doc[name] = value
        else:
            super(BaseDocument,self).__setattr__(name,value)

    def process_doc_kwargs(self,kwargs):
        doc = {}
        for key,prop in list(self._base_properties.items()):
            try:
                value = prop.get_python_value(kwargs.get(key) or prop.get_default_value())
            except ValueError:
                value = kwargs.get(key) or prop.get_default_value()
            if value:
                doc[key] = value
        for i in BUILTIN_DOC_ATTRS:
            if kwargs.get(i):
                doc[i] = kwargs[i]
        return doc

    def set_pk(self, pk):
        self._doc['_id'] = pk
        self._id = pk
        self.id = self._db.parse_id(pk)
        self.pk = self.id

    def get_indexed_props(self):
        index_list = []
        for key,prop in list(self._base_properties.items()):
            if prop.index == True:
                index_list.append(key)
        return index_list

    def get_unique_props(self):
        unique_list = []
        for key, prop in list(self._base_properties.items()):
            if prop.unique == True:
                unique_list.append(key)
        return unique_list

    def get_indexes(self):
        index_list = []
        for i in self.get_indexed_props():
            try:
                index_list.append(self.get_index_name(i,self._doc[i]))
            except KeyError:
                pass
        return index_list

    @classmethod
    def get_db(cls):
        raise NotImplementedError

    @classmethod
    def objects(cls):
        return QueryManager(cls)

    def get_doc_id(self,id):
        return '{0}:id:{1}'.format(self.__class__.__name__.lower(),id)

    def get_class_name(self):
        return self.__class__.__name__.lower()

    def get_index_name(self, prop, index_value):
        return '{0}:indexes:{1}:{2}'.format(
            self.get_class_name(), prop, index_value).lower()

    #Basic Operations

    @classmethod
    def get(cls,doc_id):
        ins = cls()
        return ins._db.get(ins,doc_id)

    @classmethod
    def all(cls):
        ins = cls()
        return ins._db.all(cls)

    def flush_db(self):
        self._db.flush_db()

    def delete(self):
        self._db.delete(self)

    def save(self):
        self._db.save(self)


    class Meta:
        use_db = 'default'
        handler = None

        
class Document(with_metaclass(DeclarativeVariablesMetaclass, BaseDocument)):
    @classmethod
    def get_db(cls):
        return cls.Meta.handler.get_db(cls.Meta.use_db)